local addonName, addon = ...

local plugin = addon:NewModule("Battleground Button", "AceHook-3.0")
plugin.Icon = [[Interface\BattlefieldFrame\UI-Battlefield-Icon]]

local AceGUI	= LibStub("AceGUI-3.0")
local FA		= LibStub("LibFrameAnchorRegistry-1.0")

local db

local SettingPoistion = false
local ChangingShow = false
local MouseIsOver = false
local ConfigMode = false

function plugin:OnInitialize()
	local defaults = {
		profile = {
			Enabled = true,
			
			Anchor = "mMinimapLFDButton",
			Point = "TOPLEFT",
			RelPoint = "BOTTOMLEFT",
			x = 0,
			y = 0,
			
			Scale = 1,
			
			Hide = false,
			ShowOnMouseOver = true,
		}
	}

	self.db = addon.db:RegisterNamespace(self:GetName(), defaults)
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "ProfileChanged")
	
	FA:RegisterAnchor("mMinimap", "Battleground Button", "mMinimapBattlegroundButton", MiniMapBattlefieldFrame)
end

function plugin:OnEnable()	
	self:SecureHook(MiniMapBattlefieldFrame, "SetPoint", "SetPosition")
	self:SecureHook(MiniMapBattlefieldFrame, "Show", "ShowFrame")
	self:HookScript(MiniMapBattlefieldFrame, "OnEnter", function()
		Minimap:GetScript("OnEnter")(Minimap)
	end)
	self:HookScript(MiniMapBattlefieldFrame, "OnLeave", function()
		Minimap:GetScript("OnLeave")(Minimap)
	end)
	
	addon.RegisterCallback(self, "MinimapEnter")
	addon.RegisterCallback(self, "MinimapLeave")
	addon.RegisterCallback(self, "ConfigMode")
	
	self:ApplySettings()
end

function plugin:OnDisable()
end

function plugin:ProfileChanged(event)
	db = self.db.profile
	self:ApplySettings()
end

function plugin:MinimapEnter()
	MouseIsOver = true
	if not MiniMapBattlefieldFrame:IsShown() and db.ShowOnMouseOver and GetBattlefieldStatus(1) ~= "none" then
		MiniMapBattlefieldFrame:Show()
	end
end

function plugin:MinimapLeave()
	MouseIsOver = false
	if MiniMapBattlefieldFrame:IsShown() and db.Hide and not ConfigMode then
		MiniMapBattlefieldFrame:Hide()
	end
end

function plugin:ShowFrame(frame)
	if ChangingShow then return end
	ChangingShow = true
	if db.Hide and not MouseIsOver and not ConfigMode then
		frame:Hide()
	else
		frame:Show()
	end
	ChangingShow = false
end

function plugin:SetPosition(frame)
	if SettingPoistion then return end
	SettingPoistion = true
	frame:SetScale(db.Scale)
	frame:ClearAllPoints()
	local anchor = FA:GetAnchor(db.Anchor)
	local s = frame:GetEffectiveScale()
	frame:SetPoint(db.Point, anchor, db.RelPoint, db.x / s, db.y / s)
	SettingPoistion = false
end

function plugin:ApplySettings()	
	MiniMapBattlefieldFrame:SetPoint(db.Point)
	MiniMapBattlefieldBorder:Hide()
	
	if db.Hide and not ConfigMode then
		MiniMapBattlefieldFrame:Hide()
	else
		MiniMapBattlefieldFrame:Show()
	end
end

function plugin:ConfigMode(event, enable)
	ConfigMode = enable
	if enable then
		MiniMapBattlefieldFrame:Show()
	elseif db.Hide or GetBattlefieldStatus(1) == "none" then
		MiniMapBattlefieldFrame:Hide()
	end
end

do
	local function Callback()
		plugin:ApplySettings()
	end
	
	function plugin:GetOptions()
		local g = AceGUI:Create("SimpleGroup")
		g:SetFullWidth(true)
		g:SetLayout("Flow")
		
		g:AddChild(UIF:CheckBox("Hide", db, "Hide", Callback, nil, 0.33))
		g:AddChild(UIF:CheckBox("Show on mouse over", db, "ShowOnMouseOver", Callback, nil, 0.33))
		
		g:AddChild(UIF:NewLine())
		
		local t = AceGUI:Create("FrameAnchorDropdown")
		t:SetRelativeWidth(0.5)
		t:SetList(FA:GetAnchors(MiniMapBattlefieldFrame))
		t:SetLabel("Anchor")
		t:SetValue(db.Anchor)
		t:SetCallback("OnValueChanged", function(self, event, cat, name, frame)
			db.x = 0
			db.y = 0
			db.Anchor = cat
			Callback()
		end)
		g:AddChild(t)
		g:AddChild(UIF:Slider("Scale", db, "Scale", 0.1, 5, 0.01, Callback, 0.5))
		g:AddChild(UIF:GridSelect("Point", db, "Point", "Anchor", 3, Callback, 0.5, 75))
		g:AddChild(UIF:GridSelect("Relative Point", db, "RelPoint", "Anchor", 3, Callback, 0.5, 75))
		g:AddChild(UIF:PositionSlider("x", "X", db, "x", 0.1, Callback, 0.44))
		g:AddChild(UIF:RelativeSpacer(0.1))
		g:AddChild(UIF:PositionSlider("y", "Y", db, "y", 0.1, Callback, 0.44))
		return g
	end
end