local addonName, addon = ...

local plugin = addon:NewModule("Boss Frames", "AceHook-3.0")
plugin.Icon = [[Interface\LootFrame\LootPanel-Icon]]

local AceGUI	= LibStub("AceGUI-3.0")
local FA		= LibStub("LibFrameAnchorRegistry-1.0")

local db

local SettingPoistion = false
local ConfigMode = false
local ConfigFrame

local Frame = CreateFrame("Frame")
Frame:SetWidth(100)
Frame:SetHeight(100)
Frame:Show()

function plugin:OnInitialize()
	local defaults = {
		profile = {
			Enabled = true,
			
			Anchor = "mMinimapQuestTracker",
			Point = "TOPRIGHT",
			RelPoint = "BOTTOMRIGHT",
			x = 0,
			y = -10,
			
			Scale = 1,
			
			Hide = false,
		}
	}

	self.db = addon.db:RegisterNamespace(self:GetName(), defaults)
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "ProfileChanged")
	
	FA:RegisterAnchor("mMinimap", "Boss Frames", "mMinimapBossFrames", Frame)
	
	ConfigFrame = WatchFrame:CreateTexture()
	ConfigFrame:SetTexture(0.5, 0, 0, 0.5)
	ConfigFrame:SetAllPoints()
	ConfigFrame:Hide()
	
	self:SetEnabledState(db.Enabled)
end

function plugin:OnEnable()	
	addon.RegisterCallback(self, "ConfigMode")
	FA.RegisterCallback(self, "FrameRegistered", "ApplySettings")
	
	for i=1, 4, 1 do
		_G["Boss"..i.."TargetFrame"]:ClearAllPoints()
		_G["Boss"..i.."TargetFrame"]:SetParent(Frame)
		_G["Boss"..i.."TargetFrame"]:SetPoint("TOP", i == 1 and Frame or _G["Boss"..(i-1).."TargetFrame"], i == 1 and "TOP" or "BOTTOM")
		_G["Boss"..i.."TargetFrame"].SetPoint = function() end
		self:SecureHook(_G["Boss"..i.."TargetFrame"], "Show", "ShowFrame")
	end
	
	self:ApplySettings()
end

function plugin:OnDisable()
	
	addon.UnregisterAllCallbacks(self)
	FA.UnregisterAllCallbacks(self)
end

function plugin:ProfileChanged(event)
	db = self.db.profile
	self:ApplySettings()
end

function plugin:ShowFrame(frame)
	if db.Hide then
		frame:Hide()
	end
end

function plugin:ApplySettings()	
	Frame:SetScale(db.Scale)	
	local anchor = FA:GetAnchor(db.Anchor)
	local s = Frame:GetEffectiveScale()
	Frame:ClearAllPoints()
	Frame:SetPoint(db.Point, anchor, db.RelPoint, db.x / s, db.y / s)	
	
	if self:IsEnabled() and not db.Enabled then self:Disable() end
	if not self:IsEnabled() and db.Enabled then self:Enable() end
end

function plugin:ConfigMode(event, enable)
	ConfigMode = enable
	if enable then
		for i=1, 4, 1 do
			_G["Boss"..i.."TargetFrame"]:Show()
		end
	else
		for i=1, 4, 1 do
			_G["Boss"..i.."TargetFrame"]:Hide()
		end
	end
end

do
	local function Callback()
		plugin:ApplySettings()
	end
	
	
	function plugin:GetOptions()
		local g = AceGUI:Create("SimpleGroup")
		g:SetFullWidth(true)
		g:SetLayout("Flow")
		
		g:AddChild(UIF:CheckBox("Enabled", db, "Enabled", Callback, "Disable this module if you have a different addon that controls the boss frame to prevent conflicts.", 0.4))
		g:AddChild(UIF:CheckBox("Hide", db, "Hide", Callback, "Hides boss frames completely. They will not show up at any point", 0.4))
		
		local t = AceGUI:Create("FrameAnchorDropdown")
		t:SetRelativeWidth(0.5)
		t:SetList(FA:GetAnchors(Frame))
		t:SetLabel("Anchor")
		t:SetValue(db.Anchor)
		t:SetCallback("OnValueChanged", function(self, event, cat, name, frame)
			db.x = 0
			db.y = 0
			db.Anchor = cat
			Callback()
		end)
		g:AddChild(t)
		g:AddChild(UIF:Slider("Scale", db, "Scale", 0.1, 5, 0.01, Callback, 0.5))
		g:AddChild(UIF:GridSelect("Point", db, "Point", "Anchor", 3, Callback, 0.5, 75))
		g:AddChild(UIF:GridSelect("Relative Point", db, "RelPoint", "Anchor", 3, Callback, 0.5, 75))
		g:AddChild(UIF:PositionSlider("x", "X", db, "x", 0.1, Callback, 0.44))
		g:AddChild(UIF:RelativeSpacer(0.1))
		g:AddChild(UIF:PositionSlider("y", "Y", db, "y", 0.1, Callback, 0.44))
		
		return g
	end
end