local addonName, addon = ...

local plugin = addon:NewModule("Capture Bar", "AceHook-3.0")
plugin.Icon = [[Interface\Icons\Achievement_BG_DefendXtowers_AV]]

local AceGUI	= LibStub("AceGUI-3.0")
local FA		= LibStub("LibFrameAnchorRegistry-1.0")

local db

local SettingPoistion = false
local ConfigMode = false
local ConfigFrame

function plugin:OnInitialize()
	local defaults = {
		profile = {
			Enabled = true,
			
			Anchor = "BlizzMinimap",
			Point = "TOP",
			RelPoint = "BOTTOM",
			x = 0,
			y = -5,
			
			Scale = 1,
		}
	}

	self.db = addon.db:RegisterNamespace(self:GetName(), defaults)
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "ProfileChanged")
		
	ConfigFrame = CaptureBar_Create(10000)
	
	FA:RegisterAnchor("mMinimap", "Capture Bar", "mMinimapCaptureBar", ConfigFrame)
end

function plugin:OnEnable()		
	
	self:SecureHook("WorldStateAlwaysUpFrame_Update", "WorldStateAlwaysUpFrame_Update")
	
	addon.RegisterCallback(self, "ConfigMode")
	FA.RegisterCallback(self, "FrameRegistered", "ApplySettings")
	
	self:ApplySettings()
end

function plugin:OnDisable()
end

function plugin:ProfileChanged(event)
	db = self.db.profile
	self:ApplySettings()
end

function plugin:SetPosition(frame)
	if SettingPoistion then return end
	SettingPoistion = true
	
	frame:SetScale(db.Scale)	
	local anchor = FA:GetAnchor(db.Anchor)
	local s = frame:GetEffectiveScale()
	frame:ClearAllPoints()
	frame:SetPoint(db.Point, anchor, db.RelPoint, db.x / s, db.y / s)
	
	SettingPoistion = false
end

function plugin:ApplySettings()	
	for i = 1, NUM_EXTENDED_UI_FRAMES do
		_G["WorldStateCaptureBar" .. i]:SetPoint("CENTER")
	end
	
	ConfigFrame:SetScale(db.Scale)
	ConfigFrame:ClearAllPoints()
	local anchor = FA:GetAnchor(db.Anchor)
	local s = ConfigFrame:GetEffectiveScale()	ConfigFrame:SetPoint(db.Point, anchor, db.RelPoint, db.x / s, db.y / s)
end

function plugin:ConfigMode(event, enable)
	ConfigMode = enable
	if enable then
		ConfigFrame:Show()
	else
		ConfigFrame:Hide()
	end
end

function plugin:WorldStateAlwaysUpFrame_Update()
	for i = 1, NUM_EXTENDED_UI_FRAMES do
		if not plugin:IsHooked(_G["WorldStateCaptureBar"..i], "SetPoint") then
			plugin:SecureHook(_G["WorldStateCaptureBar"..i], "SetPoint", "SetPosition")
			FA:RegisterAnchor("mMinimap", "Capture Bar "..i, "mMinimapCaptureBar"..i, _G["WorldStateCaptureBar"..i])
			_G["WorldStateCaptureBar"..i]:SetPoint("CENTER")
		end
	end
end

do
	local function Callback()
		plugin:ApplySettings()
	end
	
	
	function plugin:GetOptions()
		local g = AceGUI:Create("SimpleGroup")
		g:SetFullWidth(true)
		g:SetLayout("Flow")
		
		local t = AceGUI:Create("FrameAnchorDropdown")
		t:SetRelativeWidth(0.5)
		t:SetList(FA:GetAnchors(ConfigFrame))
		t:SetLabel("Anchor")
		t:SetValue(db.Anchor)
		t:SetCallback("OnValueChanged", function(self, event, cat, name, frame)
			db.x = 0
			db.y = 0
			db.Anchor = cat
			Callback()
		end)
		g:AddChild(t)
		g:AddChild(UIF:Slider("Scale", db, "Scale", 0.1, 5, 0.01, Callback, 0.5))
		g:AddChild(UIF:GridSelect("Point", db, "Point", "Anchor", 3, Callback, 0.5, 75))
		g:AddChild(UIF:GridSelect("Relative Point", db, "RelPoint", "Anchor", 3, Callback, 0.5, 75))
		g:AddChild(UIF:PositionSlider("x", "X", db, "x", 0.1, Callback, 0.44))
		g:AddChild(UIF:RelativeSpacer(0.1))
		g:AddChild(UIF:PositionSlider("y", "Y", db, "y", 0.1, Callback, 0.44))
		
		return g
	end
end