local addonName, addon = ...

local plugin = addon:NewModule("Mail", "AceHook-3.0")
plugin.Icon = [[Interface\FriendsFrame\FriendsFrameScrollIcon]]

local AceGUI	= LibStub("AceGUI-3.0")
local FA		= LibStub("LibFrameAnchorRegistry-1.0")
local LSM		= LibStub("LibSharedMedia-3.0")

local db

local SettingPoistion = false
local ConfigMode = false
local ConfigFrame

function plugin:OnInitialize()
	local defaults = {
		profile = {
			Enabled = true,
			
			Anchor = "BlizzMinimap",
			Point = "BOTTOMLEFT",
			RelPoint = "BOTTOMLEFT",
			x = 0,
			y = -5,
			
			Scale = 1,
			
			FontSettings = {
				Font = "Default",
				Size = 12,
				Flags = "OUTLINE",
				Color = {1, 1, 1, 1},
			},
			
			Hide = false,
			ShowAsText = false,
		}
	}

	self.db = addon.db:RegisterNamespace(self:GetName(), defaults)
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "ProfileChanged")
	
	FA:RegisterAnchor("mMinimap", "Mail", "mMinimapMail", MiniMapMailFrame)
	
	MiniMapMailFrame.Text = MiniMapMailFrame:CreateFontString(nil, "ARTOWRK", "GameFontNormal")
	MiniMapMailFrame.Text:SetPoint("CENTER")
	MiniMapMailFrame.Text:SetText("New Mail")
end

function plugin:OnEnable()		
	
	self:SecureHook(MiniMapMailFrame, "SetPoint", "SetPosition")
	self:SecureHook(MiniMapMailFrame, "Show", "Show")
	
	addon.RegisterCallback(self, "ConfigMode")
	FA.RegisterCallback(self, "FrameRegistered", "ApplySettings")
	
	self:ApplySettings()
end

function plugin:OnDisable()
end

function plugin:ProfileChanged(event)
	db = self.db.profile
	self:ApplySettings()
end

function plugin:Show(fram)
	
end

function plugin:SetPosition(frame)
	if SettingPoistion then return end
	SettingPoistion = true
	
	frame:SetScale(db.Scale)	
	local anchor = FA:GetAnchor(db.Anchor)
	local s = frame:GetEffectiveScale()
	frame:ClearAllPoints()
	frame:SetPoint(db.Point, anchor, db.RelPoint, db.x / s, db.y / s)
	
	SettingPoistion = false
end

function plugin:ApplySettings()	
	MiniMapMailBorder:Hide()
	MiniMapMailIcon:SetTexCoord(0.02, 0.92, 0.02, 0.92)
	MiniMapMailFrame:SetPoint("CENTER")
	
	MiniMapMailFrame.Text:SetFont(LSM:Fetch("font", db.FontSettings.Font), db.FontSettings.Size, db.FontSettings.Flags)
	MiniMapMailFrame.Text:SetTextColor(unpack(db.FontSettings.Color))
	
	if db.ShowAsText then
		MiniMapMailIcon:Hide()
		MiniMapMailFrame.Text:Show()
		MiniMapMailFrame:SetWidth(MiniMapMailFrame.Text:GetStringWidth())
	else
		MiniMapMailIcon:Show()
		MiniMapMailFrame.Text:Hide()
		MiniMapMailFrame:SetWidth(33)
	end
	
	if (db.Hide or not GetLatestThreeSenders()) or not ConfigMode then
		MiniMapMailFrame:Hide()
	else
		MiniMapMailFrame:Show()
	end
end

function plugin:ConfigMode(event, enable)
	ConfigMode = enable
	if enable then
		MiniMapMailFrame:Show()
	elseif db.Hide or not GetLatestThreeSenders() then
		MiniMapMailFrame:Hide()
	end
end

do
	local function Callback()
		plugin:ApplySettings()
	end
	
	FontFlagList = {
		["NONE"] = "None",
		["OUTLINE"] = "Outline",
		["THICKOUTLINE"] =  "Thick Outline",
		["MONOCHROME"] = "Monochrome",
		["OUTLINEMONOCHROME"] = "Outline Monochrome",
	}
	
	function plugin:GetOptions()
		local g = AceGUI:Create("SimpleGroup")
		g:SetFullWidth(true)
		g:SetLayout("Flow")
		
		g:AddChild(UIF:CheckBox("Hide", db, "Hide", Callback, nil, 0.33))
		g:AddChild(UIF:CheckBox("Show as Text", db, "ShowAsText", Callback, "Will hide the icon and show 'New Mail' instead.", 0.33))
		
		local t = AceGUI:Create("FrameAnchorDropdown")
		t:SetRelativeWidth(0.5)
		t:SetList(FA:GetAnchors(MiniMapMailFrame))
		t:SetLabel("Anchor")
		t:SetValue(db.Anchor)
		t:SetCallback("OnValueChanged", function(self, event, cat, name, frame)
			db.x = 0
			db.y = 0
			db.Anchor = cat
			Callback()
		end)
		g:AddChild(t)
		g:AddChild(UIF:Slider("Scale", db, "Scale", 0.1, 5, 0.01, Callback, 0.5))
		g:AddChild(UIF:GridSelect("Point", db, "Point", "Anchor", 3, Callback, 0.5, 75))
		g:AddChild(UIF:GridSelect("Relative Point", db, "RelPoint", "Anchor", 3, Callback, 0.5, 75))
		g:AddChild(UIF:PositionSlider("x", "X", db, "x", 0.1, Callback, 0.44))
		g:AddChild(UIF:RelativeSpacer(0.1))
		g:AddChild(UIF:PositionSlider("y", "Y", db, "y", 0.1, Callback, 0.44))
		
		g:AddChild(UIF:NewLine(15))
		
		g:AddChild(UIF:FontSettings("Font settings", db.FontSettings, true, Callback, 1))
		
		return g
	end
end