local addonName, addon = ...

local plugin = addon:NewModule("Vehicle Seats", "AceHook-3.0")
plugin.Icon = [[Interface\Vehicles\SeatIndicator\vehicle-mammoth]]

local AceGUI	= LibStub("AceGUI-3.0")
local FA		= LibStub("LibFrameAnchorRegistry-1.0")

local db

TempDB = {
}

local SettingPoistion = false
local ConfigMode = false

function plugin:OnInitialize()
	local defaults = {
		profile = {
			Enabled = true,
			
			Anchor = "mMinimapDurability",
			Point = "BOTTOM",
			RelPoint = "TOP",
			x = 0,
			y = 0,
			Scale = 1.0,
		}
	}

	self.db = addon.db:RegisterNamespace(self:GetName(), defaults)
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "ProfileChanged")
	
	FA:RegisterAnchor("mMinimap", "Vehicle Seat Indicator", "mMinimapVehicleSeatIndicator", VehicleSeatIndicator)
end

function plugin:OnEnable()	
	UIF = LibStub("LibGUIFactory-1.0"):GetFactory("mMinimap")
	addon.RegisterCallback(self, "ConfigMode")
	
	self:SecureHook(VehicleSeatIndicator, "SetPoint", "SetPosition")
	
	self:ApplySettings()
end

function plugin:OnDisable()
	addon.UnregisterAllCallbacks(self)
end

function plugin:ProfileChanged(event)
	db = self.db.profile
	self:ApplySettings()
end

function plugin:SetPosition(...)
	if SettingPoistion then return end
	SettingPoistion = true
	VehicleSeatIndicator:SetScale(db.Scale)
	VehicleSeatIndicator:ClearAllPoints()
	local anchor = FA:GetAnchor(db.Anchor)
	local s = VehicleSeatIndicator:GetEffectiveScale()
	VehicleSeatIndicator:SetPoint(db.Point, anchor, db.RelPoint, db.x / s, db.y / s)	
	SettingPoistion = false
end

function plugin:ApplySettings()
	VehicleSeatIndicator:SetPoint(db.Point)	
end

function plugin:Show()
	VehicleSeatIndicator:SetHeight(128)
	VehicleSeatIndicator:SetWidth(128)
	VehicleSeatIndicatorBackgroundTexture:SetTexture([[Interface\Vehicles\SeatIndicator\vehicle-mammoth]])
	VehicleSeatIndicator:Show()
	TempDB.Showing = true
end

function plugin:Hide()
	VehicleSeatIndicator_UnloadTextures()
	TempDB.Showing = nil
end

function plugin:ConfigMode(event, enable)
	ConfigMode = enable
	if enable then
		self:Show()
	else
		self:Hide()
	end
end

do
	local function Callback()
		plugin:ApplySettings()
	end
	
	function plugin:GetOptions()
		local g = AceGUI:Create("SimpleGroup")
		g:SetFullWidth(true)
		g:SetLayout("Flow")
		
		g:AddChild(UIF:NewLine())
		
		local t = AceGUI:Create("FrameAnchorDropdown")
		t:SetRelativeWidth(0.5)
		t:SetList(FA:GetAnchors(VehicleSeatIndicator))
		t:SetLabel("Anchor")
		t:SetValue(db.Anchor)
		t:SetCallback("OnValueChanged", function(self, event, cat, name, frame)
			db.x = 0
			db.y = 0
			db.Anchor = cat
			Callback()
		end)
		g:AddChild(t)
		g:AddChild(UIF:Slider("Scale", db, "Scale", 0.1, 5, 0.01, Callback, 0.5))
		g:AddChild(UIF:GridSelect("Point", db, "Point", "Anchor", 3, Callback, 0.5, 75))
		g:AddChild(UIF:GridSelect("Relative Point", db, "RelPoint", "Anchor", 3, Callback, 0.5, 75))
		g:AddChild(UIF:PositionSlider("x", "X", db, "x", 0.1, Callback, 0.44))
		g:AddChild(UIF:RelativeSpacer(0.1))
		g:AddChild(UIF:PositionSlider("y", "Y", db, "y", 0.1, Callback, 0.44))
		return g
	end
end