---------------------------------------------------
-- Config
---------------------------------------------------
local LSM = LibStub("LibSharedMedia-3.0")
local AceGUI = LibStub("AceGUI-3.0")
local OptionsFrame
local db
local SelectedOption, OptionsStatus = nil, {selected = "ShowAndHide"}
local UIF
local mMinimapIcon = [[Interface\Icons\INV_Offhand_Sunwell_D_01]]

local ConfigMode = false


function mMinimap:ChangeCallback(widget, event, ...)
	if widget and not widget.GetUserData then
		widget = self
		self = mMinimap
		if widget:GetUserData("key") == "MinimapPositionX" or widget:GetUserData("key") == "MinimapPositionY" then
			db.MinimapPositionSet = true
		end
	end		
	self:ShowHideFrames()
	self:Update()
end

function mMinimap:ToggleConfigIfVisible()
	if OptionsFrame and OptionsFrame.frame:IsShown() then
		self:ToggleConfig()
		self:ToggleConfig()
	end
end

local OptionsStatus = {
	width = 800,
	height = 600,
}

function mMinimap:ToggleConfig()
	UIF = LibStub("LibGUIFactory-1.0"):GetFactory("mMinimap")
	db = self.db.profile
	if not OptionsFrame then	
		OptionsFrame = UIF:Frame("mMinimap")
		OptionsFrame:SetStatusTable(OptionsStatus)
		OptionsFrame:SetLayout("Flow")
		OptionsFrame:Hide()			
		OptionsFrame:SetCallback("OnClose", function(widget)				
			OptionsFrame:ReleaseChildren()	
			OptionsFrame:Release()
			OptionsFrame = nil
		end)			
	end
	
	if OptionsFrame.frame:IsShown() then
		OptionsFrame:Hide()
	else			
		OptionsFrame:AddChild(UIF:Button("Toggle Config Mode", function()
			mMinimap.ConfigMode = not mMinimap.ConfigMode
			mMinimap.callbacks:Fire("ConfigMode", mMinimap.ConfigMode)
		end))
	
		m = UIF:TreeGroup()
		m:SetFullHeight(true)
		m:SetFullWidth(true)
		m:SetLayout("Fill")	
		m:SetCallback("OnGroupSelected", function(self, event, group) SelectedOption = group end)									
		OptionsFrame:AddChild(m)
		
		-- Toggles
		m:AddGroup({{text = "Info", value = "Info"}},
			function()				
				local text = UIF:NormalText(
[[- Right click will show the tracking menu.
- Middle click shows the calendar.
- Hover over to show hidden buttons.]]) 
				text:SetImage(mMinimapIcon)
				local group = UIF:InlineGroup("Information")
				group:AddChild(text)					
				
				return group
			end)				
		
		
		for name, module in self:IterateModules() do
			m:AddGroup({{text = "Modules", value = "Modules"}, {text = name, value = name, order = module.Order }}, 
			function() 
				local t = UIF:Title1(name)
				t:SetImage(module.Icon)
				t:SetImageSize(40, 40)
				
				return t, UIF:NewLine(15), module.GetOptions()
			end, module.Icon)
		end
		
		
		-- Profile
		m:AddGroup({{text = "Profile", value = "Profile"}},
			function()
				return UIF:AceProfileOption(mMinimap.db)
			end)			
						
		m:SetSelected(SelectedOption or "Info")
		m:SetStatusTable(OptionsStatus)	
		OptionsFrame:Show()
	end
end
