local addonName, addon = ...
mMinimap = LibStub("AceAddon-3.0"):NewAddon(addon, "mMinimap", "AceEvent-3.0", "AceConsole-3.0", "AceHook-3.0")
local mMinimap = mMinimap

mMinimap.callbacks = LibStub("CallbackHandler-1.0"):New(mMinimap)

-- Lib
local LSM = LibStub("LibSharedMedia-3.0")

-- Locals
local db
local mMinimapIcon = [[Interface\Icons\INV_Offhand_Sunwell_D_01]]

mMinimap.ConfigMode = false

function mMinimap:OnInitialize()
	local defaults = {
		profile = {
			Enabled = true,
		}
	}
	
	self.db  = LibStub("AceDB-3.0"):New("mMinimapDB", defaults, "Default")
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "ProfileChanged")
	
	self:SetupLDB()
	
	self:SetEnabledState(db.Enabled)
	
	self:HookScript(Minimap, "OnEnter", function() addon.callbacks:Fire("MinimapEnter") end)
	self:HookScript(Minimap, "OnLeave", function()
		local x, y = GetCursorPosition()
		local minx = Minimap:GetLeft()
		local maxx = Minimap:GetRight()
		local miny = Minimap:GetBottom()
		local maxy = Minimap:GetTop()
		local s = Minimap:GetEffectiveScale()
		x, y = x / s, y / s
		
		--if x < minx or x > maxx or y < miny or y > maxy then
			addon.callbacks:Fire("MinimapLeave") 
		--end
	end)
	Minimap:SetScript("OnMouseUp", function(self, button) 
		if (button == 'RightButton') then
			ToggleDropDownMenu(1, nil, MiniMapTrackingDropDown, self, 0, 0)
		elseif (button == "MiddleButton") then
			ToggleCalendar()
		else
			Minimap_OnClick(self, button)
		end
		addon.callbacks:Fire("MinimapClick", button)
	end)
	
	-- Border
	MinimapBorder:SetTexture(nil)
	MinimapBorderTop:Hide()	
	-- Compas
	MinimapNorthTag:SetAlpha(0)	
	
	-- Register slash command
	self:RegisterChatCommand("mm", "ToggleConfig")
	
	local UIFSettings = {
		font = LibStub("LibSharedMedia-3.0"):Fetch("font", "Prototype"),
		fontSize = 16,			
	}
	local UIF = LibStub("LibGUIFactory-1.0"):GetFactory("mMinimap", UIFSettings)
end

function mMinimap:OnEnable()		
end

function mMinimap:OnDisable()
end

function mMinimap:ProfileChanged(...)
	db = self.db.profile
end

function mMinimap:SetupLDB()
	local LDB = LibStub("LibDataBroker-1.1", true)
	if not LDB then return end
	
	local l = LDB:NewDataObject("mMinimap")
	l.type = "launcher"
	l.icon = mMinimapIcon
	l.OnClick = function(self, button)
		if button == "LeftButton" then
			mMinimap:ToggleConfig()
		elseif button == "RightButton" then
			mMinimap.ConfigMode = not mMinimap.ConfigMode
			mMinimap.callbacks:Fire("ConfigMode", mMinimap.ConfigMode)
		end
	end
	l.OnTooltipShow = function(tt)
		tt:AddLine("m|cffe6b155M|rinimap")
		tt:AddLine("|cffffffffLeft Click:|r Toggle configuration")
		tt:AddLine("|cffffffffRight Click:|r Toggle configuration mode")
	end
	
	self.ldbojb = l
end